#!/usr/bin/perl

require "./stat.pl";

# Each line contains stats for a trial of experiment
# Prints the following fields in each line

# filename 
# experiment location 
# true separation distance between beacon and listener (in cm)
# total samples collected
# mean distance   (in cm)
# mode distance   (in cm)
# median distance (in cm)
# min distance    (in cm)
# max distance    (in cm)
# stddev          (in cm)
# 100*stddev/mean (in cm)	   
# percent error from true distance (measure-true)/true

##### main 

@letter = ( "A", "B", "C" );
@places = ( "ConfRoomCarpet", "ConfRoomChair", "ConfRoomTable" );

for($i = 0; $i < 11; $i++) {
    for($j = 0; $j <= $#letter; $j++) {
	$num = $i;
	if ($num < 10) {
	    $num = "0$num";
	}
	$filenamesuf = "$num$letter[$j].log";
	if((-e ($filename = "Trial$filenamesuf")) ||
	   (-e ($filename = "trial$filenamesuf"))) {
	    printf("$filename $places[$j]\t%d ", $i*30);
	    proc_file($filename, $i*30);
	}
    }
}

@infCorridor = ( 292, 252, 228, 209 );

for($i = 0; $i <= $#infCorridor; $i++) {
    $num = $i + 1;
    $filename = "trial0$num\D.log";
    printf("$filename InfiniteCorridor\t%d ", $infCorridor[$i]);
    proc_file($filename, $infCorridor[$i]);
}

for($i = 11; $i <= 22; $i++) {
    $filename = "Trial$i"."E.log";
    printf("$filename ElevtrLobbyCarpet\t%d ", $i*30);
    proc_file($filename, $i*30);
}

##### niam

sub proc_file {
    my($filename) = $_[0];
    my($correct_dist) = $_[1];
    my($est_dist) = 0;
    my($totalSamples) = 0;
    my($time, $type, $seqno, $resv, $space, $id, $dist, $coord, $orient );
    my(@samples);

    open(FILE, "$filename") || die "cannot open $filename\n";

    while ($aline = <FILE>) {
	chop($aline);
	if($aline) {
	    ($time, $type, $seqno, $resv, $space, $id, $dist, $coord, $orient )
		= split(" ", $aline);
	    if(should_include_sample( $dist, $filename )) {
		push(@samples, $dist);
	    }
	}
    }

    $totalSamples = $#samples + 1;
    printf("$totalSamples ");
#    print_stats( \&identity, \&identity, \@samples );
    $est_dist = print_stats( \&b2cm, \&b2cm_zero, \@samples );

    if($correct_dist != 0) {
	printf(" %f\n", 100*($correct_dist - $est_dist)/$correct_dist);
    }
    else {
	printf("\n");
    }

    close(FILE);
}

sub should_include_sample {
    local($dist) = $_[0];
    local($file) = $_[1];

    if($file eq "Trial06C.log" ||
       $file eq "Trial07C.log" ||
       $file eq "Trial08C.log") {
	if($dist > 200) {
	    return 0;
	}
    }
    elsif($file eq "Trial19E.log") {
	if($dist > 350) {
	    return 0;
	}
    }
    elsif($file eq "Trial22E.log") {
	if($dist > 350 || $dist < 200) {
	    return 0;
	}
    }
    return 1;
}

sub b2cm
{
    return b2cm_zero(@_) - 79.370496;
}

sub b2cm_zero
{
    local(@bodhi) = @_;
    return $bodhi[0] * 34449/15625;
}

sub identity {
    return @_;
}

# mean mode median min max stddev
sub print_stats {
    local($conv) = $_[0];
    local($conv_zero) = $_[1];
    local($els)  = $_[2];
    local($mean) = 0;
    local($stddev) = 0;

    $mean = &$conv(&perf_mean(@$els));
    $stddev = &$conv_zero(&perf_stddev(@$els));

    printf("%f %f %f %f %f %f %f", 
	   $mean,
	   &$conv(&perf_mode(@$els)),
	   &$conv(&perf_median(@$els)),
	   &$conv(&perf_min(@$els)), 
	   &$conv(&perf_max(@$els)),
	   $stddev,
	   100*$stddev/$mean);

    return $mean;
}

