
#
#
# Courtesy of Dave Andersen.
#
#

##
# Now compute some factoids about them.
# Let's split this out in to a library later so all of our analysis
# routines can use them.
##

sub identity {
    return @_;
}

sub print_all_stats {
    local(@els) = @_;
    &print_all_stats_conv( \&identity, \&identity, \@els );
}

sub print_all_stats_conv {
    local($conv) = $_[0];
    local($conv_zero) = $_[1];
    local($els)  = $_[2];
    printf("  Mean:            %f\n", &$conv(&perf_mean(@$els)));
    printf("  Mode:            %f\n", &$conv(&perf_mode(@$els)));
    printf("  Trimmed mean:    %f\n", &$conv(&perf_trimmed_mean(@$els)));
    printf("  Median:          %f\n", &$conv(&perf_median(@$els)));
    printf("  Min/Max:         %f  /  %f\n", &$conv(&perf_min(@$els)), &$conv(&perf_max(@$els)));
    printf("  Stddev:          %f\n", &$conv_zero(&perf_stddev(@$els)));
    printf("  Trimmed Stddev:  %f\n", &$conv_zero(&perf_trimmed_stddev(@$els)));
}

sub perf_mean {
	local(@items) = @_;
	local($count) = 0;
	local($sum) = 0;

	foreach $item (@items) {
		$sum += $item;
		$count++;
	}
	return 0 if !$count;
	($sum/$count);
}

sub bynum { $a <=> $b; }

sub perf_trimmed_mean {
	local(@items) = sort bynum @_;
	local($count) = 0;
	local($sum) = 0;
	local($i);

	for ($i = 1; $i < $#items; $i++) {
		$sum += $items[$i];
		$count++;
	}
	return 0 if !$count;
	($sum/$count);
}

sub perf_min {
	local(@sorted) = sort bynum @_;
	local($count) = $#sorted;

	return $sorted[0];
}

sub perf_max {
	local(@sorted) = sort bynum @_;
	local($count) = $#sorted;

	return $sorted[$count];
}

sub perf_median {
	local(@sorted) = sort bynum @_;
	local($count) = $#sorted;
	local($median);

	return 0 if !$#sorted;

	$count = int($count/2);
	if ($count % 2) {
		$median = ( $sorted[$count] + $sorted[$count + 1]) / 2;
	} else {
		$median = $sorted[$count];
	}
	return $median;
}

sub perf_stddev {
	local(@items) = @_;
	local($mean) = &perf_mean(@items);
	local($n) = $#items + 1;
	local($variance) = 0;
	local($i) = 0;

	for ($i = 0; $i < $n; $i++) {
		$variance += ($items[$i] - $mean) * ($items[$i] - $mean);
	}

	if ($#items == 0) {
	  return 0;
	}
	if ($n <= 1) { return 0; }
	if ($variance >= 0) { return sqrt($variance/($n-1)); }
	return sqrt(-$variance/($n-1));
}

sub perf_trimmed_stddev {
	local(@items) = sort bynum @_;
	local($mean) = &perf_mean(@items);
	local($n) = $#items + 1;
	local($variance) = 0;
	local($i) = 0;

	for ($i = 1; $i < $n-1; $i++) {
		$variance += ($items[$i] - $mean) * ($items[$i] - $mean);
	}

	if ($n <= 3) { return 0; }
	if ($variance >= 0) { return sqrt($variance/($n-3)); }
	return sqrt(-$variance/($n-3));
}

# Courtesy of Allen Miu
sub perf_mode {
    local(@items)    = sort bynum @_;
    local($prevDist) = $items[0];
    local($freq)     = 0;
    local(@modeDists)= ();
    local($modeFreq) = 0;
    local($sum)      = 0;

    foreach $item (@items) {
	if($prevDist == $item) {
	    $freq++;
	}
	else {
	    if($freq > $modeFreq) {
		@modeDists = ();
		$modeFreq = $freq;
	    }
	    if($freq >= $modeFreq) {
		push(@modeDists, $prevDist);
		$prevDist = $item;
		$freq = 1;
	    }
	}
    }
    if($freq > 1) { 
	# run this block only if the "else" clause did not get run
	# in the for-loop
	if($freq > $modeFreq) {
	    @modeDists = ();
	    $modeFreq = $freq;
	}
	if($freq >= $modeFreq) {
	    push(@modeDists, $prevDist);
	}
    }
    
    foreach $dist (@modeDists) {
	$sum += $dist;
    }
    
    return $sum/($#modeDists + 1);
}

1;
